@@ -206,6 +206,7 @@ class _TransientRunConfigSchema(_TransientConfigSchema):
     copy_in_before = fields.List(fields.Str(), missing=[])
     copy_out_after = fields.List(fields.Str(), missing=[])
     copy_timeout = fields.Int(allow_none=True)
+    rsync = fields.Bool(missing=False)
     prepare_only = fields.Bool(missing=False)
     qemu_bin_name = fields.Str(allow_none=True)
     qemu_args = fields.List(fields.Str(), missing=[])
@@ -219,6 +220,7 @@ class _TransientRunConfigSchema(_TransientConfigSchema):
     ssh_user = fields.Str(missing="vagrant", allow_none=True)
     ssh_console = fields.Bool(missing=False)
     ssh_with_serial = fields.Bool(missing=False)
+    ssh_option = fields.List(fields.Str(), missing=[])
     shared_folder = fields.List(fields.Str(), missing=[])
     sftp_bin_name = fields.Str(missing="sftp-server", allow_none=True)
     no_virtio_scsi = fields.Bool(missing=False)
