@@ -35,6 +35,7 @@ class SshConfig:
         user: Optional[str] = None,
         password: Optional[str] = None,
         args: Optional[List[str]] = None,
+        extra_options: Optional[List[str]] = None,
     ) -> None:
         self.host = host
         self.port = port if port is not None else 22
@@ -46,6 +47,9 @@ class SshConfig:
 
         # Pass these as default args
         self.args.extend(self.__default_ssh_args())
+        if extra_options:
+            for opt in extra_options:
+                self.args.extend(["-o", opt])
 
     def __default_ssh_args(self) -> List[str]:
         return [
@@ -198,6 +202,31 @@ def _prepare_builtin_keys() -> List[str]:
     return list(builtins.values())
 
 
+def find_ssh_port_forward(qmp_client: qemu.QmpClient) -> int:
+    # Use qmp to determine what port was selected by the kernel
+    resp = qmp_client.send_sync(
+        {
+            "execute": "human-monitor-command",
+            "arguments": {"command-line": "info usernet"},
+        }
+    )["return"]
+
+    for line in resp.split("\n"):
+        match = re.match(
+            r"""
+            \s+TCP\[HOST_FORWARD\]  # Match only a HOST_FORWARD line
+            (?:\s+\S+){,2}          # Skip the first two groups
+            \s+(\d+)                # Capture the host port being forwarded
+            \s+\S+\s+22\s+          # But only match if the destination is 22
+            """,
+            line,
+            re.VERBOSE,
+        )
+        if match is not None:
+            return int(match.group(1))
+    raise RuntimeError("Unable to locate SSH port")
+
+
 def scp(
     source: str,
     destination: str,
@@ -233,26 +262,39 @@ def scp(
         )
 
 
-def find_ssh_port_forward(qmp_client: qemu.QmpClient) -> int:
-    # Use qmp to determine what port was selected by the kernel
-    resp = qmp_client.send_sync(
-        {
-            "execute": "human-monitor-command",
-            "arguments": {"command-line": "info usernet"},
-        }
-    )["return"]
+def rsync(
+    source: str,
+    destination: str,
+    config: SshConfig,
+    copy_from: bool = False,
+    capture_stdout: bool = False,
+    capture_stderr: bool = False,
+) -> Tuple[Optional[str], Optional[str]]:
+    if config.user is not None:
+        host = f"{config.user}@{config.host}"
+    else:
+        host = f"{config.host}"
 
-    for line in resp.split("\n"):
-        match = re.match(
-            r"""
-            \s+TCP\[HOST_FORWARD\]  # Match only a HOST_FORWARD line
-            (?:\s+\S+){,2}          # Skip the first two groups
-            \s+(\d+)                # Capture the host port being forwarded
-            \s+\S+\s+22\s+          # But only match if the destination is 22
-            """,
-            line,
-            re.VERBOSE,
+    ssh_command = [config.ssh_bin_name] + config.args + ["-p", str(config.port)]
+    rsync_args = ["--archive", "--acls", "--xattrs", "--sparse", "--info=progress2"]
+
+    priv_keys = _prepare_builtin_keys()
+    for key in priv_keys:
+        ssh_command.extend(["-i", key])
+
+    rsync_args.extend(["-e", " ".join(ssh_command)])
+
+    if copy_from is False:
+        host += f":{destination}"
+        return utils.run_check_retcode(
+            ["rsync", *rsync_args, source, host],
+            capture_stdout=capture_stdout,
+            capture_stderr=capture_stderr,
+        )
+    else:
+        host += f":{source}"
+        return utils.run_check_retcode(
+            ["rsync", *rsync_args, host, destination],
+            capture_stdout=capture_stdout,
+            capture_stderr=capture_stderr,
         )
-        if match is not None:
-            return int(match.group(1))
-    raise RuntimeError("Unable to locate SSH port")
