@@ -219,7 +219,7 @@ class TransientVm:
             new_args.extend(["-device", "virtio-scsi-pci,id=scsi"])
             for idx, image in enumerate(self.vm_images):
                 new_args.extend(["-drive", f"file={image.path},if=none,id=hd{idx}"])
-                new_args.extend(["-device", f"scsi-hd,drive=hd{idx}"])
+                new_args.extend(["-device", f"scsi-hd,drive=hd{idx},bootindex={idx}"])
 
         if self.__needs_ssh():
             if self.__needs_ssh_console():
@@ -462,7 +462,7 @@ class TransientVm:
 
                 # If we didn't reach the expected shutdown, this will terminte
                 # the VM. Otherwise, this does nothing.
-                self.qemu_runner.terminate()
+                self.qemu_runner.terminate(kill_after=self.config.shutdown_timeout)
 
             # Note that we always return the SSH exit code, even if the guest failed to
             # shut down. This ensures the shutdown_timeout=0 case is handled as expected.
